@@ -5,15 +5,10 @@ nepali_datetime.
 
 Supports >= Python3.5
 """
-# TODO: make nepali_datetime 's "date", "datetime" objects hashable & add pickling support
-# TODO: improve documentation
-# TODO: feature to allow inject custom month names
-# TODO: more tests
+__version__ = "1.0.5"
 
 __author__ = "Amit Garu <amitgaru2@gmail.com>"
 
-__version__ = "1.0.4"
-
 import sys
 import csv
 import time as _time
@@ -36,11 +31,21 @@ _DAYNAMES = [None, "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun"]
 _FULLDAYNAMES = [None, "Monday", "Tueday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday"]
 
 _STRFTIME_CUSTOM_MAP = {
-    'b': lambda x: '%s' % _MONTHNAMES[x.tm_mon],
-    'B': lambda x: '%s' % _FULLMONTHNAMES[x.tm_mon],
-    'N': lambda x: '%s' % _MONTHNAMES_NP[x.tm_mon],
-    'a': lambda x: '%s' % _DAYNAMES[(x.tm_wday % 7) or 7],
-    'A': lambda x: '%s' % _FULLDAYNAMES[(x.tm_wday % 7) or 7]
+    'a': lambda o: '%s' % _DAYNAMES[(o.weekday() % 7) or 7],
+    'A': lambda o: '%s' % _FULLDAYNAMES[(o.weekday() % 7) or 7],
+    'w': lambda o: '%d' % o.weekday(),
+    'd': lambda o: '%02d' % o.day,
+    'b': lambda o: '%s' % _MONTHNAMES[o.month],
+    'B': lambda o: '%s' % _FULLMONTHNAMES[o.month],
+    'N': lambda o: '%s' % _MONTHNAMES_NP[o.month],
+    'm': lambda o: '%02d' % o.month,
+    'y': lambda o: '%02d' % (o.year % 100),
+    'Y': lambda o: '%d' % o.year,
+    'H': lambda o: '%02d' % getattr(o, 'hour', 0),
+    'I': lambda o: '%02d' % (getattr(o, 'hour', 0) % 12,),
+    'p': lambda o: 'AM' if getattr(o, 'hour', 0) < 12 else 'PM',
+    'M': lambda o: '%02d' % getattr(o, 'minute', 0),
+    'S': lambda o: '%02d' % getattr(o, 'second', 0),
 }
 
 _CALENDAR = {}
@@ -89,8 +94,7 @@ def _wrap_strftime(object, format, timetuple):
                 i += 1
                 if ch == 'f':
                     if freplace is None:
-                        freplace = '%06d' % getattr(object,
-                                                    'microsecond', 0)
+                        freplace = '%06d' % getattr(object, 'microsecond', 0)
                     newformat.append(freplace)
                 elif ch == 'z':
                     if zreplace is None:
@@ -117,8 +121,8 @@ def _wrap_strftime(object, format, timetuple):
                                 # strftime is going to have at this: escape %
                                 Zreplace = s.replace('%', '%%')
                     newformat.append(Zreplace)
-                elif ch in ('a', 'A', 'b', 'B', 'N'):
-                    newformat.append(_STRFTIME_CUSTOM_MAP[ch](timetuple))
+                elif ch in ('a', 'A', 'w', 'd', 'b', 'B', 'N', 'm', 'y', 'Y', 'H', 'I', 'p', 'M', 'S'):
+                    newformat.append(_STRFTIME_CUSTOM_MAP[ch](object))
                 else:
                     push('%')
                     push(ch)
@@ -127,7 +131,7 @@ def _wrap_strftime(object, format, timetuple):
         else:
             push(ch)
     newformat = "".join(newformat)
-    return _time.strftime(newformat, timetuple)
+    return newformat
 
 
 def _bin_search(key, *arr):
@@ -372,14 +376,10 @@ class date:
 
     def calendar(self, justify=4):
         format_str = '{:>%s}' % justify
-        _found_today = False
 
-        def _check_today_day(day, week_start_day, cal):
-            nonlocal _found_today
-            if not _found_today and day < week_start_day:
-                indx = cal[-1].index(format_str.format(day))
-                cal[-1][indx] = '\033[31m{}\033[39m'.format(cal[-1][indx])
-                _found_today = True
+        def _mark_today(indx):
+            indx_day = cal[indx].index(format_str.format(self.day))
+            cal[indx][indx_day] = '\033[31m{}\033[39m'.format(cal[indx][indx_day])
 
         total_days_month = _days_in_month(self.year, self.month)
         start_weekday = self.__class__(self.year, self.month, 1).weekday()
@@ -387,18 +387,25 @@ class date:
                [format_str.format('Sun'), *(format_str.format(j) for j in _DAYNAMES[1:-1])],
                [format_str.format(' ') for _ in range(start_weekday)]]
         cal[-1].extend([format_str.format(j) for j in range(1, 8 - start_weekday)])
-        week_start_day = int(cal[-1][-1]) + 1
-        _check_today_day(self.day, week_start_day, cal)
+        cal_cursor = 8 - start_weekday
+        cal_range = [(1, 7 - start_weekday)]
 
-        for i in range((total_days_month - week_start_day) // 7):
-            cal.append([format_str.format(j) for j in range(week_start_day, week_start_day + 7)])
-            week_start_day = int(cal[-1][-1]) + 1
-            _check_today_day(self.day, week_start_day, cal)
+        total_mid_weeks = (total_days_month - cal_cursor) // 7
+        for i in range(total_mid_weeks):
+            cal_range.append((cal_cursor, cal_cursor + 6))
+            cal.append([format_str.format(j) for j in range(cal_cursor, cal_cursor + 7)])
+            cal_cursor += 7
 
-        if int(cal[-1][-1]) < total_days_month:
-            week_start_day = int(cal[-1][-1]) + 1
-            cal.append([format_str.format(j) for j in range(week_start_day, total_days_month + 1)])
-            _check_today_day(self.day, week_start_day, cal)
+        if cal_cursor < total_days_month:
+            cal.append([format_str.format(j) for j in range(cal_cursor, total_days_month + 1)])
+            cal_range.append((cal_cursor, total_days_month))
+
+        if sys.platform.startswith('linux'):
+            # currently only linux supported
+            for i, cr in enumerate(cal_range):
+                if cr[0] <= self.day <= cr[1]:
+                    _mark_today(-len(cal_range) + i)
+                    break
 
         cal = '\n' + '\n'.join(' '.join(j) for j in cal) + '\n\n'
         sys.stdout.write(cal)
